#define COMMAND_H

/****************************************************************

    Copyright (c) 1996-1997 Fred Pesch

    This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License as published by the Free Software 
    Foundation (version 2 of the License).

    This program is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the 
    implied warranty of MERCHANTABILITY or FITNESS FOR A 
    PARTICULAR PURPOSE.  See the GNU General Public License 
    for more details.

    For a copy of the GNU General Public License, write to the 
    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
    MA 02139, USA.

****************************************************************/

#include <Xm/Xm.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <sys/types.h>

#include "dialogs.h"
#include "fileview.h"
#include "error_handler.h"
#include "config.h"
#include "interface.h"
#include "fof.h"
#include "offix_dnd.h"
#include "shell.h"

typedef struct {
  int nb_commands;
  char **command;
  char **interactivestring;
  char *headerstring;
  int interactive;
  int usercommand;
} TExecList;

TExecList ActualExecList;
int NbCommandProcessed;
int ChildErrorRaised;
int ChildExist=0;
pid_t ChildPID;
struct sigaction ChildAction;
struct sigaction OldAction;

XtInputId InputId;
Widget W_InputTextField;

void UpdateStatusDialog(void)
{
  Widget w_temp;
  XmString s_temp;

  /* Setup the Status Dialog */
  w_temp = XtNameToWidget(W_StatusDialog, "StatusLabel1");
  s_temp = XmStringCreateSimple(ActualExecList.headerstring);
  if(s_temp!=NULL && XmStringLength(s_temp)>0)
    XtVaSetValues(w_temp, XmNlabelString, s_temp, NULL);
  XmStringFree(s_temp);
  /*  w_temp = XtNameToWidget(W_StatusDialog, "StatusLabel2");
  s_temp = XmStringCreateSimple(ActualExecList.interactivestring[NbCommandProcessed]);
  XtVaSetValues(w_temp, XmNlabelString, s_temp, NULL);
  XmStringFree(s_temp);*/
  w_temp = XtNameToWidget(W_StatusDialog, "StatusScale");
  XtVaSetValues(w_temp, XmNvalue, NbCommandProcessed, NULL);
}

void GetCommandResult( XtPointer clientData, int *fid, XtInputId *id)
{
  char buf[BUFSIZ+1];
  int nbbytes;

  nbbytes=read(*fid, buf, BUFSIZ);

  if(nbbytes!=0) {
    buf[nbbytes]=(char)0;
    XmTextInsert(W_InputTextField, XmTextGetLastPosition(W_InputTextField), buf);
  }
}

void InitStatusDialog(int max_commands)
{
  Widget w_temp;

  w_temp = XtNameToWidget(W_StatusDialog, "StatusScale");
  XtVaSetValues(w_temp, XmNvalue, 0, XmNminimum, 0, XmNmaximum, max_commands, NULL);
  W_InputTextField = XtNameToWidget(W_StatusDialog, "*StatusText");
  XmTextSetString(W_InputTextField, "");
  XmTextSetInsertionPosition(W_InputTextField, 0);

  w_temp = XtNameToWidget(W_StatusDialog, "StatusOkButton");
  XtVaSetValues(w_temp, XmNsensitive, False, NULL);

}

extern void GeneralExecCommand(void);

void ChildHandler(int sig)
{
  int i;
  int status;
  Widget w_temp;
  Bool debug_mode;
  int result;

  if(!ChildExist) return;
  GetVarBool("DebugMode", &debug_mode);
  if(debug_mode) fprintf(stderr,"\nreceived child signal");

  waitpid(ChildPID, &status, 0);
  if(status!=0) {
    ChildErrorRaised=1;
  }

  NbCommandProcessed++;
  UpdateStatusDialog();
  if(NbCommandProcessed>=ActualExecList.nb_commands) {

    w_temp=XtNameToWidget(W_StatusDialog, "*StatusStopButton");
    XtVaSetValues(w_temp, XmNsensitive, False, NULL);


    if(debug_mode) fprintf(stderr,"\nmain execution finished");
      for(i=0;i<ActualExecList.nb_commands;i++) {
	if(ActualExecList.interactive) XtFree(ActualExecList.interactivestring[i]);
	XtFree(ActualExecList.command[i]);
      }
      XtFree(ActualExecList.headerstring);
      XtFree((char *) ActualExecList.command);

      XtRemoveInput(InputId);
      if(!ChildErrorRaised) {
	XtUnmanageChild(W_StatusDialog);
	XmUpdateDisplay(W_MainWindow);
      }
      else {
	w_temp = XtNameToWidget(W_StatusDialog, "*StatusOkButton");
	XtVaSetValues(w_temp, XmNsensitive, True, NULL);
	ChildErrorRaised=0;
      }
      result=sigaction(SIGCHLD, &OldAction, NULL);
      if(result==-1) FatalError("command.c", "ChildHandler","Unable to install the default child signal handler");
      ChildExist=0;

      /*      AskRefreshFileList=1;*/

      if(ActualExecList.interactive) XtFree((char *) ActualExecList.interactivestring);
  }
  else {
    if(debug_mode) fprintf(stderr,"\nnext command");
    GeneralExecCommand();
  }
}

/* main exec command (status dialog) */
void GeneralExecCommand(void)
{
  pid_t child;
  int result;
  char *cmdstr;
  char *arglist[256];
  int countarg;
  int pipe_to_parent[2];
  Bool debug_mode;
  Widget w_temp;

  GetVarBool("DebugMode", &debug_mode);
  if(!debug_mode) pipe(pipe_to_parent);

  UpdateStatusDialog();

  w_temp=XtNameToWidget(W_StatusDialog, "*StatusStopButton");
  XtVaSetValues(w_temp, XmNsensitive, True, NULL);

  /* first install the child handler */
  ChildAction.sa_handler=ChildHandler;
  result=sigaction(SIGCHLD, &ChildAction, /* added */ &OldAction);
  if(result==-1) FatalError("command.c", "GeneralExecCommand","Unable to install signal handler");
  child=fork();
  if(child==-1) FatalError("command.c", "GeneralExecCommand","unable to fork()");
  if(child==0) {
    if(debug_mode) {    
      fprintf(stderr,"\nI'm in the child process");
      cmdstr=XtMalloc(strlen(ActualExecList.command[NbCommandProcessed])+1);
      strcpy(cmdstr,ActualExecList.command[NbCommandProcessed]);
      fprintf(stderr,"\ncommand : %s", cmdstr);
      countarg=0;
      arglist[countarg]=strtok(cmdstr," ");
      fprintf(stderr,"\narg[%d]=%s",countarg, arglist[countarg]);
      while(arglist[countarg]!=NULL) {
	countarg++;
	arglist[countarg]=strtok(NULL, " ");
	fprintf(stderr,"\narg[%d]=%s",countarg, arglist[countarg]);
      }
      /*added*/      XtFree(cmdstr);
      sleep(2);
      fprintf(stderr,"\nChild is terminated");
      exit(0);
    } else {
      cmdstr=XtMalloc(strlen(ActualExecList.command[NbCommandProcessed])+1);
      strcpy(cmdstr,ActualExecList.command[NbCommandProcessed]);
      countarg=0;
      arglist[countarg]=strtok(cmdstr," ");
      while(arglist[countarg]!=NULL) {
	countarg++;
	arglist[countarg]=strtok(NULL, " ");
      }
      /* added  XtFree(cmdstr); (error) */

      close(pipe_to_parent[0]);
      result=dup2(pipe_to_parent[1],fileno(stdout));
      result=dup2(pipe_to_parent[1],fileno(stderr));
      if(NbCommandProcessed==0) sleep(1);
      execvp(arglist[0], arglist);
      fprintf(stderr,"\nChild exec error : %s\n",arglist[0]);
      exit(EXIT_FAILURE);
    }
  }

  ChildPID=child;

  if(!debug_mode) {
    result=dup2(pipe_to_parent[0],fileno(stdin));
    close(pipe_to_parent[1]);
  }
}

/* execute command dispatcher */
void GeneralExecCommand2(TExecList execlist)
{
  int result;
  int i;
  Bool debug_mode;

  GetVarBool("DebugMode", &debug_mode);
  /* interactive command execution */
  if(execlist.interactive) {
    for(i=0;i<execlist.nb_commands;i++) {
      if(ConfirmDialog("Execute Command",execlist.interactivestring[i])) {
	if(debug_mode) fprintf(stderr,"\nDebug Mode command :\n\n%s\n",execlist.command[i]);
	else {
	  result=system(execlist.command[i]);   
	  if(result==127) FatalError("command.c","GeneralExecCommand","Cannot find /bin/sh");
	  else if(result==-1) MessageDialog("Exec problem", "I've received an error return for the last command...");
	}
      }
    }
    AskRefreshFileList=1;
    /* destroy ExecList entries */
    for(i=0;i<execlist.nb_commands;i++) {
      if(execlist.interactive) XtFree(execlist.interactivestring[i]);
      XtFree(execlist.command[i]);
    }
    XtFree(execlist.headerstring);
    XtFree((char *) execlist.command);
    if(execlist.interactive) XtFree((char *) execlist.interactivestring);

    /* non interactive mode */
    } else if(execlist.usercommand) {
    char *execcommand;

    for(i=0;i<execlist.nb_commands;i++) {
      if(debug_mode) fprintf(stderr,"\nDebug Mode command :\n\n%s\n",execlist.command[i]);
      else {
	execcommand=XtMalloc(strlen(execlist.command[i])+3);
	strcpy(execcommand, execlist.command[i]);
	XtFree(execlist.command[i]);
	strcat(execcommand, " &");
	result=system(execcommand);
	XtFree(execcommand);
	if(result==127) FatalError("command.c","GeneralExecCommand","Cannot find /bin/sh");
	else if(result==-1) MessageDialog("Exec problem", "I've received an error return for the last command...");
      }
    }

    XtFree(execlist.headerstring);
    XtFree((char *) execlist.command);
    if(execlist.interactive) XtFree((char *) execlist.interactivestring);
  } else { /* non user command */
    ActualExecList.nb_commands=execlist.nb_commands;
    ActualExecList.command=execlist.command;
    ActualExecList.interactivestring=execlist.interactivestring;
    ActualExecList.headerstring=execlist.headerstring;
    ChildErrorRaised=0;
    NbCommandProcessed=0;
    InputId=XtAppAddInput(AppContext, fileno(stdin), (XtPointer) XtInputReadMask, GetCommandResult, NULL);
    InitStatusDialog(execlist.nb_commands);
    XtManageChild(W_StatusDialog);
    XmUpdateDisplay(W_MainWindow);
    ChildExist=1;
    GeneralExecCommand();
  }
}


void FileCopyCommandRoutine(char *SourceDirectory, char *DestDirectory, TCopyFlags flags)
{
  TExecList execlist;
  TViewNode *actual;
  int count;
  int i;

  /* first count entries */
  count = 0;
  actual = SelectFileViewList;
  while(actual!=NULL) {
    actual=actual->next;
      count++;
  }

  /* title */
  execlist.headerstring = XtMalloc(strlen("Copying files...")+1);
  strcpy(execlist.headerstring, "Copying files...");

  /* allocate command list strings */
  execlist.nb_commands = count;
  execlist.command = (char **) XtMalloc(sizeof(char *)*count);

  if(flags.interactive) {
    execlist.interactive=1;
    execlist.interactivestring = (char **) XtMalloc(sizeof(char *)*count);
  } else execlist.interactive=0;

  i=0;
  actual = SelectFileViewList;
  while(actual!=NULL) {
    if(flags.interactive) {
      execlist.interactivestring[i] = (char *) XtMalloc(sizeof(char)*(strlen(actual->filenode->file.name)+1+25));
      strcpy(execlist.interactivestring[i], "Confirm copying file :\n\n");
      strcat(execlist.interactivestring[i], actual->filenode->file.name);
    }

    execlist.command[i]= XtMalloc(7);
    strcpy(execlist.command[i], "cp -f ");
  
    /* Make option list */
    if(flags.recurse) {
      execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1 + 3);
      strcat(execlist.command[i], "-r ");
    }

    if(flags.preserve_access) {
      execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1 + 3);
      strcat(execlist.command[i], "-p ");
    }

    if(flags.preserve_links) {
      execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1 + 3);
      strcat(execlist.command[i], "-d ");
    }

    if(flags.update) {
      execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1 + 3);
      strcat(execlist.command[i], "-u ");
    }
    
    if(flags.hardlinks) {
      execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1 + 3);
      strcat(execlist.command[i], "-l ");
    }
    
    if(flags.symlinks) {
      execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1 + 3);
      strcat(execlist.command[i], "-s ");
    }
    
    if(flags.one_filesystem) {
      execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1 + 3);
      strcat(execlist.command[i], "-x ");
    }
    
    /* Create source filenames */
    execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1 + strlen(SourceDirectory) + 1 + strlen(actual->filenode->file.name)+1);
    strcat(execlist.command[i], SourceDirectory);
    strcat(execlist.command[i], "/");
    strcat(execlist.command[i], actual->filenode->file.name);
    strcat(execlist.command[i], " ");
	

    /* Create Destination directory */
    
    execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1 + strlen(DestDirectory));
    strcat(execlist.command[i], DestDirectory);

    actual=actual->next; 
    i++;
  }	

  execlist.usercommand=0;
  GeneralExecCommand2(execlist);
}
  
void FileMoveCommandRoutine(char *SourceDirectory, char *DestDirectory, TMoveFlags flags)
{
  TExecList execlist;
  TViewNode *actual;
  int count;
  int i;

  /* first count entries */
  count = 0;
  actual = SelectFileViewList;
  while(actual!=NULL) {
    actual=actual->next;
      count++;
  }

  /* title */
  execlist.headerstring = XtMalloc(strlen("Moving files...")+1);
  strcpy(execlist.headerstring, "Moving files...");

  /* allocate command list strings */
  execlist.nb_commands = count;
  execlist.command = (char **) XtMalloc(sizeof(char *)*count);

  if(flags.interactive) execlist.interactive=1; else execlist.interactive=0;

  if(flags.interactive) {
    execlist.interactive=1;
    execlist.interactivestring = (char **) XtMalloc(sizeof(char *)*count);
  } else execlist.interactive=0;

  i=0;
  actual = SelectFileViewList;
  while(actual!=NULL) {
    if(flags.interactive) {
      execlist.interactivestring[i] = (char *) XtMalloc(sizeof(char)*(strlen(actual->filenode->file.name)+1+24));
      strcpy(execlist.interactivestring[i], "Confirm Moving file :\n\n");
      strcat(execlist.interactivestring[i], actual->filenode->file.name);
    }

    execlist.command[i]= XtMalloc(7);
    strcpy(execlist.command[i], "mv -f ");
  
    /* Make option list */

    if(flags.update) {
      execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1 + 3);
      strcat(execlist.command[i], "-u ");
    }	
    /* Create source filenames */
    execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1 + strlen(SourceDirectory) + 1 + strlen(actual->filenode->file.name)+1);
    strcat(execlist.command[i], SourceDirectory);
    strcat(execlist.command[i], "/");
    strcat(execlist.command[i], actual->filenode->file.name);
    strcat(execlist.command[i], " ");
    
    /* Create Destination directory */

    execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1 + strlen(DestDirectory));
    strcat(execlist.command[i], DestDirectory);

    actual=actual->next;
    i++;
  }

  execlist.usercommand=0;
  GeneralExecCommand2(execlist);
}

void FileDeleteCommandRoutine(char *SourceDirectory, TDeleteFlags flags)
{
  TExecList execlist;
  TViewNode *actual;
  int count;
  int i;

  /* first count entries */
  count = 0;
  actual = SelectFileViewList;
  while(actual!=NULL) {
    actual=actual->next;
      count++;
  }

  /* title */
  execlist.headerstring = XtMalloc(strlen("Deleting files...")+1);
  strcpy(execlist.headerstring, "Deleting files...");

  /* allocate command list strings */
  execlist.nb_commands = count;
  execlist.command = (char **) XtMalloc(sizeof(char *)*count);

  if(flags.interactive) execlist.interactive=1; else execlist.interactive=0;

  if(flags.interactive) {
    execlist.interactive=1;
    execlist.interactivestring = (char **) XtMalloc(sizeof(char *)*count);
  } else execlist.interactive=0;

  i=0;
  actual = SelectFileViewList;
  while(actual!=NULL) {
    if(flags.interactive) {
      execlist.interactivestring[i] = (char *) XtMalloc(sizeof(char)*(strlen(actual->filenode->file.name)+1+25));
      strcpy(execlist.interactivestring[i], "Confirm Deleting file :\n\n");
      strcat(execlist.interactivestring[i], actual->filenode->file.name);
    }

    execlist.command[i]= XtMalloc(7);
    strcpy(execlist.command[i], "rm -f ");
  
    /* Make option list */

    if(flags.recurse) {
      execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1 + 3);
      strcat(execlist.command[i], "-r ");
    }

    /* Create source filenames */

    execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1 + strlen(SourceDirectory) + 1 + strlen(actual->filenode->file.name)+1);
    strcat(execlist.command[i], SourceDirectory);
    strcat(execlist.command[i], "/");
    strcat(execlist.command[i], actual->filenode->file.name);
    strcat(execlist.command[i], " ");

    actual=actual->next;
    i++;
  }

  execlist.usercommand=0;
  GeneralExecCommand2(execlist);

}

void FileRenameCommandRoutine(char *SourceFile, char *DestFile)
{
  TExecList execlist;

  execlist.interactive=0;

  execlist.command = (char **) XtMalloc(sizeof(char *) * 1);

  execlist.nb_commands = 1;

  execlist.headerstring = XtMalloc(strlen("Rename file")+1);
  strcpy(execlist.headerstring, "Rename File");

  execlist.command[0] = XtMalloc(sizeof(char)*(strlen(SourceFile)+strlen(DestFile)+11));

  execlist.usercommand=0;
  strcpy(execlist.command[0], "mv -f -- ");
  strcat(execlist.command[0], SourceFile);
  strcat(execlist.command[0], " ");
  strcat(execlist.command[0], DestFile);

  GeneralExecCommand2(execlist);
}

void FileAttributesCommandRoutine(char *SourceFile, TAttributesFlags flags)
{
  char *commandstring;
  TExecList execlist;

  commandstring = XtMalloc(sizeof(char)*(strlen(SourceFile)+7+40));
  
  strcpy(commandstring, "chmod ");

  strcat(commandstring,"u");
  if(flags.user_read) strcat(commandstring,"+r");else strcat(commandstring,"-r");
  strcat(commandstring,",u");
  if(flags.user_write) strcat(commandstring,"+w");else strcat(commandstring,"-w");
  strcat(commandstring,",u");
  if(flags.user_exec) strcat(commandstring,"+x");else strcat(commandstring,"-x");

  strcat(commandstring,",g");
  if(flags.group_read) strcat(commandstring,"+r");else strcat(commandstring,"-r");
  strcat(commandstring,",g");
  if(flags.group_write) strcat(commandstring,"+w");else strcat(commandstring,"-w");
  strcat(commandstring,",g");
  if(flags.group_exec) strcat(commandstring,"+x");else strcat(commandstring,"-x");

  strcat(commandstring,",o");
  if(flags.others_read) strcat(commandstring,"+r");else strcat(commandstring,"-r");
  strcat(commandstring,",o");
  if(flags.others_write) strcat(commandstring,"+w");else strcat(commandstring,"-w");
  strcat(commandstring,",o");
  if(flags.others_exec) strcat(commandstring,"+x ");else strcat(commandstring,"-x ");
  strcat(commandstring, "-- ");
  strcat(commandstring, SourceFile);

  execlist.interactive=0;
  execlist.nb_commands=1;
  execlist.headerstring = XtMalloc(strlen("Change File Attributes")+1);
  strcpy(execlist.headerstring, "Change File Attributes");
  execlist.command = (char **) XtMalloc(sizeof(char *));
  execlist.command[0]=commandstring;

  execlist.usercommand=0;
  GeneralExecCommand2(execlist);
}

void FileCreateCommandRoutine(char *DestFile, Bool makedir)
{
  char *commandstring;
  TExecList execlist;

  if(!makedir) {
    commandstring = XtMalloc(sizeof(char)*(strlen(DestFile)+10));
  
    strcpy(commandstring, "touch -- ");
    strcat(commandstring, DestFile);
  
    execlist.interactive=0;
    execlist.nb_commands=1;
    execlist.headerstring = XtMalloc(strlen("Create File")+1);
    strcpy(execlist.headerstring, "Create File");
    execlist.command = (char **) XtMalloc(sizeof(char *));
    execlist.command[0]=commandstring;

    execlist.usercommand=1;
    GeneralExecCommand2(execlist);
  } else {
    commandstring = XtMalloc(sizeof(char)*(strlen(DestFile)+10));
  
    strcpy(commandstring, "mkdir -- ");
    strcat(commandstring, DestFile);
  
    execlist.interactive=0;
    execlist.nb_commands=1;
    execlist.headerstring = XtMalloc(strlen("Create Directory")+1);
    strcpy(execlist.headerstring, "Create Directory");
    execlist.command = (char **) XtMalloc(sizeof(char *));
    execlist.command[0]=commandstring;

    execlist.usercommand=1;
    GeneralExecCommand2(execlist);
  }

}

void FileExecCommandRoutine(char *program, char *arguments, Bool needxterm)
{
  TExecList execlist;
  char *commandstring;
  char *xterm_arguments;

  GetVarString("XtermArguments", &xterm_arguments);

  if(needxterm) {
    commandstring = XtMalloc(strlen(xterm_arguments)+1+3+strlen(program)+1+strlen(arguments)+1);
    strcpy(commandstring, xterm_arguments);
    strcat(commandstring, " -e ");
    strcat(commandstring, program);
    strcat(commandstring, " ");
    strcat(commandstring, arguments);

    execlist.interactive=0;
    execlist.nb_commands=1;
    execlist.headerstring = XtMalloc(strlen("Execute in xterm")+1);
    strcpy(execlist.headerstring, "Execute in xterm");
    execlist.command = (char **) XtMalloc(sizeof(char *));
    execlist.command[0]=commandstring;
    
    execlist.usercommand=1;
    GeneralExecCommand2(execlist);
    
  } else {
    commandstring = XtMalloc(strlen(program)+1+strlen(arguments)+1);
    strcpy(commandstring, program);
    strcat(commandstring, " ");
    strcat(commandstring, arguments);
    
    execlist.interactive=0;
    execlist.nb_commands=1;
    execlist.headerstring = XtMalloc(strlen("Execute")+1);
    strcpy(execlist.headerstring, "Execute");
    execlist.command = (char **) XtMalloc(sizeof(char *));
    execlist.command[0]=commandstring;
    
    
    execlist.usercommand=1;
    GeneralExecCommand2(execlist);

  }

}

void FileViewCommandRoutine(void)
{
  char *commandstring;
  TExecList execlist;
  char *viewcommand;

  /* try to get the view command in the format list, use the default one else */

  viewcommand=GetFormatViewCommand(SelectFileViewList->filenode->file.name);

  if(viewcommand==NULL || strcmp(viewcommand,"")==0)
    GetVarString("DefaultViewCommand",&viewcommand);
  

  commandstring = XtMalloc(strlen(viewcommand)+1+strlen(SelectFileViewList->filenode->file.name)+1);
  
  strcpy(commandstring, viewcommand);
  strcat(commandstring, " ");
  strcat(commandstring, SelectFileViewList->filenode->file.name);

  execlist.interactive=0;
  execlist.nb_commands=1;
  execlist.headerstring = XtMalloc(strlen("View file")+1);
  strcpy(execlist.headerstring, "View File");
  execlist.command = (char **) XtMalloc(sizeof(char *));
  execlist.command[0]=commandstring;

  execlist.usercommand=1;
  GeneralExecCommand2(execlist);
}

void FileEditCommandRoutine(void)
{
  char *commandstring;
  TExecList execlist;
  char *editcommand;

  /* Get the edit command */

  /* try to get the view command in the format list, use the default one else */

  editcommand=GetFormatEditCommand(SelectFileViewList->filenode->file.name);

  if(editcommand==NULL || strcmp(editcommand,"")==0)
    GetVarString("DefaultEditCommand",&editcommand);

  commandstring = XtMalloc(strlen(editcommand)+1+strlen(SelectFileViewList->filenode->file.name)+1);
  
  strcpy(commandstring, editcommand);
  strcat(commandstring, " ");
  strcat(commandstring, SelectFileViewList->filenode->file.name);

  execlist.interactive=0;
  execlist.nb_commands=1;
  execlist.headerstring = XtMalloc(strlen("Edit File")+1);
  strcpy(execlist.headerstring, "Edit File");
  execlist.command = (char **) XtMalloc(sizeof(char *));
  execlist.command[0]=commandstring;
  
  execlist.usercommand=1;
  GeneralExecCommand2(execlist);
}

void ArchiveTarCommandRoutine(char *DestFile, TTarFlags flags)
{
  char *commandstring;
  TViewNode *actual;
  char numstring[20];
  TExecList execlist;

  commandstring = XtMalloc(sizeof(char)*8);
  strcpy(commandstring, "tar -c ");
    
  /* Make option list */
  
  if(flags.gzip) {
    commandstring = XtRealloc(commandstring, strlen(commandstring)+1 + 3);
    strcat(commandstring, "-z ");
  }
  
  if(flags.absolute) {
    commandstring = XtRealloc(commandstring, strlen(commandstring)+1 + 3);
    strcat(commandstring, "-P ");
  }
  
  if(flags.dereference) {
    commandstring = XtRealloc(commandstring, strlen(commandstring)+1 + 3);
    strcat(commandstring, "-h ");
  }

  if(flags.preserve) {
    commandstring = XtRealloc(commandstring, strlen(commandstring)+1 + 3);
    strcat(commandstring, "-p ");
  }
  
  if(flags.one_filesystem) {
    commandstring = XtRealloc(commandstring, strlen(commandstring)+1 + 3);
    strcat(commandstring, "-l ");
  }
  
  if(flags.block_size) {
    commandstring = XtRealloc(commandstring, strlen(commandstring)+1 + 3);
    strcat(commandstring, "-b ");
    sprintf(numstring, "%d ", flags.block_size_value);
    commandstring = XtRealloc(commandstring, strlen(commandstring)+1+strlen(numstring)+2);
    strcat(commandstring, numstring);
  }
  
  commandstring = XtRealloc(commandstring, strlen(commandstring)+1+3);
  strcat(commandstring, "-f ");
  
  /* Create dest filename */
  
  commandstring = XtRealloc(commandstring, strlen(commandstring)+1+strlen(DestFile)+1);
  
  strcat(commandstring, DestFile);
  strcat(commandstring, " ");
  
  /* Create Source Filenames */

  actual= SelectFileViewList;
  while(actual!=NULL) {
    commandstring = XtRealloc(commandstring, strlen(commandstring)+1 + strlen(actual->filenode->file.name)+1);
    strcat(commandstring, actual->filenode->file.name);
    strcat(commandstring, " ");
    actual=actual->next;
  }
  
  execlist.interactive=0;
  execlist.nb_commands=1;
  execlist.headerstring = XtMalloc(strlen("Archive files")+1);
  strcpy(execlist.headerstring, "Archive files");
  execlist.command = (char **) XtMalloc(sizeof(char *));
  execlist.command[0]=commandstring;
    
  execlist.usercommand=0;
  GeneralExecCommand2(execlist);
}

void ArchiveUnTarCommandRoutine(char *DestFile, Bool gunzip)
{
  char *commandstring;
  TExecList execlist;

  if(!gunzip) {
    commandstring = XtMalloc(sizeof(char)*(14+strlen(DestFile)));
    strcpy(commandstring, "tar -x -f ");
        
    /* Create source filename */

    strcat(commandstring, DestFile);
    
    execlist.interactive=0;
    execlist.nb_commands=1;
    execlist.headerstring = XtMalloc(strlen("Unarchive files")+1);
    strcpy(execlist.headerstring, "Unarchive files");
    execlist.command = (char **) XtMalloc(sizeof(char *));
    execlist.command[0]=commandstring;
    
    execlist.usercommand=0;
    GeneralExecCommand2(execlist);
  } else {

    commandstring = XtMalloc(sizeof(char)*(17+strlen(DestFile)));
    strcpy(commandstring, "tar -x -z -f ");
        
    /* Create source filename */

    strcat(commandstring, DestFile);
    
    execlist.interactive=0;
    execlist.nb_commands=1;
    execlist.headerstring = XtMalloc(strlen("Unarchive files")+1);
    strcpy(execlist.headerstring, "Unarchive files");
    execlist.command = (char **) XtMalloc(sizeof(char *));
    execlist.command[0]=commandstring;
    
    execlist.usercommand=0;
    GeneralExecCommand2(execlist);
  }
}

void ArchiveGZipCommandRoutine(TGZipFlags flags)
{
  TExecList execlist;
  TViewNode *actual;
  int count;
  int i;
  char numstring[20];

  /* first count entries */
  count = 0;
  actual = SelectFileViewList;
  while(actual!=NULL) {
    actual=actual->next;
      count++;
  }

  /* title */
  execlist.headerstring = XtMalloc(strlen("Gzip files...")+1);
  strcpy(execlist.headerstring, "Gzip files...");

  /* allocate command list strings */
  execlist.nb_commands = count;
  execlist.command = (char **) XtMalloc(sizeof(char *)*count);

  if(flags.interactive) execlist.interactive=1; else execlist.interactive=0;

  if(flags.interactive) {
    execlist.interactive=1;
    execlist.interactivestring = (char **) XtMalloc(sizeof(char *)*count);
  } else execlist.interactive=0;

  i=0;
  actual = SelectFileViewList;
  while(actual!=NULL) {
    if(flags.interactive) {
      execlist.interactivestring[i] = (char *) XtMalloc(sizeof(char)*(strlen(actual->filenode->file.name)+1+24));
      strcpy(execlist.interactivestring[i], "Confirm gzip file :\n\n");
      strcat(execlist.interactivestring[i], actual->filenode->file.name);
    }

    execlist.command[i]= XtMalloc(6);
    strcpy(execlist.command[i], "gzip ");
	
    /* Make option list */
	
    if(flags.recurse) {
      execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1 + 3);
      strcat(execlist.command[i], "-r ");
    }
    
    sprintf(numstring, "-%1d ", flags.quality);
    
    execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1+strlen(numstring));
    
    strcat(execlist.command[i], numstring);
    
    /* Create source filenames */
    
    execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1+ strlen(actual->filenode->file.name)+4);
    strcat(execlist.command[i], "-- ");
    strcat(execlist.command[i], actual->filenode->file.name);
    strcat(execlist.command[i], " ");
	
    actual=actual->next;
    i++;
  }
  
  execlist.usercommand=0;
  GeneralExecCommand2(execlist);
}

void ArchiveGUnZipCommandRoutine(TGZipFlags flags)
{
  TExecList execlist;
  TViewNode *actual;
  int count;
  int i;

  /* first count entries */
  count = 0;
  actual = SelectFileViewList;
  while(actual!=NULL) {
    actual=actual->next;
      count++;
  }

  /* title */
  execlist.headerstring = XtMalloc(strlen("Gunzip files...")+1);
  strcpy(execlist.headerstring, "Gunzip files...");

  /* allocate command list strings */
  execlist.nb_commands = count;
  execlist.command = (char **) XtMalloc(sizeof(char *)*count);

  if(flags.interactive) execlist.interactive=1; else execlist.interactive=0;

  if(flags.interactive) {
    execlist.interactive=1;
    execlist.interactivestring = (char **) XtMalloc(sizeof(char *)*count);
  } else execlist.interactive=0;

  i=0;
  actual = SelectFileViewList;
  while(actual!=NULL) {
    if(flags.interactive) {
      execlist.interactivestring[i] = (char *) XtMalloc(sizeof(char)*(strlen(actual->filenode->file.name)+1+24));
      strcpy(execlist.interactivestring[i], "Confirm gunzip file :\n\n");
      strcat(execlist.interactivestring[i], actual->filenode->file.name);
    }

    execlist.command[i]= XtMalloc(8);
    strcpy(execlist.command[i], "gunzip ");

    /* Make option list */
	
    if(flags.recurse) {
      execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+1 + 3);
      strcat(execlist.command[i], "-r ");
    }

    /* Create source filenames */
    
    execlist.command[i] = XtRealloc(execlist.command[i], strlen(execlist.command[i])+4+ strlen(actual->filenode->file.name)+1);
    strcat(execlist.command[i], "-- ");
    strcat(execlist.command[i], actual->filenode->file.name);
    strcat(execlist.command[i], " ");
	
    
    actual=actual->next;
    i++;
  }
  
  execlist.usercommand=0;
  GeneralExecCommand2(execlist);
}

void ArchiveEncodeCommandRoutine(char *NameFile, char *DestFile)
{
  char *commandstring;
  TExecList execlist;

  commandstring = XtMalloc(sizeof(char)*(13+strlen(SelectFileViewList->filenode->file.name)+1+strlen(NameFile)+3+strlen(DestFile)));
  strcpy(commandstring, "uuencode ");
    
  /* Create source filename */

  strcat(commandstring, SelectFileViewList->filenode->file.name);
  strcat(commandstring, " ");
  strcat(commandstring, NameFile);
  strcat(commandstring, " > ");

  strcat(commandstring, DestFile);

  execlist.interactive=0;
  execlist.nb_commands=1;
  execlist.headerstring = XtMalloc(strlen("UUEncode file")+1);
  strcpy(execlist.headerstring, "UUEncode file");
  execlist.command = (char **) XtMalloc(sizeof(char *));
  execlist.command[0]=commandstring;
    
  execlist.usercommand=1;
  GeneralExecCommand2(execlist);
}

void ArchiveDecodeCommandRoutine(char *DestFile)
{
  char *commandstring;
  TExecList execlist;

  commandstring = XtMalloc(sizeof(char)*(14+strlen(DestFile)));
  strcpy(commandstring, "uudecode ");
    

  /* Create source filename */

  strcat(commandstring, DestFile);

  execlist.interactive=0;
  execlist.nb_commands=1;
  execlist.headerstring = XtMalloc(strlen("UUDecode file")+1);
  strcpy(execlist.headerstring, "UUDecode file");
  execlist.command = (char **) XtMalloc(sizeof(char *));
  execlist.command[0]=commandstring;
  
  execlist.usercommand=0;
  GeneralExecCommand2(execlist);
}

void ArchiveSplitCommandRoutine(TSplitFlags flags)
{
  char *commandstring;
  char sizestr[256];
  TExecList execlist;

  sprintf(sizestr, "%d", flags.size);
  commandstring = XtMalloc(13+strlen(sizestr)+1+strlen(SelectFileViewList->filenode->file.name));
  strcpy(commandstring, "split -b ");
  strcat(commandstring, sizestr);
  strcat(commandstring, " -- ");
  strcat(commandstring, SelectFileViewList->filenode->file.name);
  if(flags.prefix) {
    commandstring = XtRealloc(commandstring, strlen(commandstring)+1+1+strlen(flags.prefix_value));
    strcat(commandstring, " ");
    strcat(commandstring, flags.prefix_value);
    XtFree(flags.prefix_value);
  }

  execlist.interactive=0;
  execlist.nb_commands=1;
  execlist.headerstring = XtMalloc(strlen("Split file")+1);
  strcpy(execlist.headerstring, "SplitFile");
  execlist.command = (char **) XtMalloc(sizeof(char *));
  execlist.command[0]=commandstring;
  
  execlist.usercommand=0;
  GeneralExecCommand2(execlist);
}

void ArchiveUnSplitCommandRoutine(int nbfiles, char **sourcefiles, char *destfile)
{
  char *commandstring;
  int totallen;
  int i;
  TExecList execlist;

  totallen = 8;
  for(i=0;i<nbfiles;i++)
    totallen+=strlen(sourcefiles[i])+1;
  totallen += 3;
  totallen+=strlen(destfile);

  commandstring = XtMalloc(totallen);
  strcpy(commandstring, "cat ");
  for(i=0;i<nbfiles;i++) {
    strcat(commandstring, sourcefiles[i]);
    strcat(commandstring, " ");
  }
  strcat(commandstring, " > ");
  strcat(commandstring, destfile);
    
  execlist.interactive=0;
  execlist.nb_commands=1;
  execlist.headerstring = XtMalloc(strlen("Unsplit files")+1);
  strcpy(execlist.headerstring, "Unsplit files");
  execlist.command = (char **) XtMalloc(sizeof(char *));
  execlist.command[0]=commandstring;
    
  execlist.usercommand=1;
  GeneralExecCommand2(execlist);

}

void DropCommandRoutine(void)
{
  TExecList execlist;
  char *string;
  int pos,i,nbcommands;

  nbcommands=0;
  string = XtMalloc(DropInfos.Size);
  pos=0;
  while(pos<DropInfos.Size) {
    i=0;
    while(DropInfos.Data[pos]!=(char)0)
      string[i++]=DropInfos.Data[pos++];
    string[i]=(char)0;
    nbcommands++;
    pos++;
  }
  XtFree(string);

  nbcommands=nbcommands-1;

  execlist.nb_commands=nbcommands;
  execlist.command = (char **) XtMalloc(sizeof(char *) * nbcommands);

  nbcommands=0;
  string = XtMalloc(DropInfos.Size);
  pos=0;
  while(pos<DropInfos.Size) {
    i=0;
    while(DropInfos.Data[pos]!=(char)0)
      string[i++]=DropInfos.Data[pos++];
    string[i]=(char)0;
    if(string[0]!=(char) 0) {
      switch(DropInfos.DropType) {
      case COPY :
	execlist.command[nbcommands]=XtMalloc(10+strlen(string)+1+strlen(DropInfos.DestDir)+1);
	strcpy(execlist.command[nbcommands], "cp -rf -- ");
	break;
      case MOVE :
	execlist.command[nbcommands]=XtMalloc(9+strlen(string)+1+strlen(DropInfos.DestDir)+1);
	strcpy(execlist.command[nbcommands], "mv -f -- ");
	break;
      case LINK :
	execlist.command[nbcommands]=XtMalloc(11+strlen(string)+1+strlen(DropInfos.DestDir)+1);
	strcpy(execlist.command[nbcommands], "cp -rsf -- ");
	break;      
      }
      strcat(execlist.command[nbcommands], string);
      strcat(execlist.command[nbcommands], " ");
      strcat(execlist.command[nbcommands], DropInfos.DestDir);
    
      nbcommands++;
    }
    pos++;
  }
  XtFree(string);

  execlist.interactive=0;
  execlist.headerstring = XtMalloc(strlen("Drop Files")+1);
  strcpy(execlist.headerstring, "Drop Files");
  execlist.usercommand = 0;

  GeneralExecCommand2(execlist);
}

/**************************** USE COMMANDS ******************************/

void UserMenuFileSubst(char **command)
{
  char *newcommand;
  char *startstr;
  char *selectionstr;
  int i,j,k;

  if(SelectFileViewList==NULL) {
    selectionstr=XtMalloc(1);
    selectionstr[0]=(char)0;
  } else {
    selectionstr = XtMalloc(strlen(SelectFileViewList->filenode->file.name)+1);
    strcpy(selectionstr, SelectFileViewList->filenode->file.name);
  }

  startstr = strstr(*command, "$FILE");
  if(startstr==NULL) {
    NonFatalError("command.c","UserMenuFileCommandRoutine","Cannot find $FILE field in User menu specification");
    return;
  }

  newcommand=XtMalloc(strlen(*command)+2+strlen(selectionstr)+1);

  /* create command string */
  for(i=0;i<((int)startstr-(int)(*command));i++) 
    newcommand[i]=(*command)[i];
  
  j=i;
  for(i=0;i<strlen(selectionstr);i++)
    newcommand[i+j]=selectionstr[i];

  k=i+j;
  for(i=0;i<(strlen(*command)-j-strlen("$FILE"));i++)
     newcommand[i+k]=(*command)[i+j+strlen("$FILE")];
  
  newcommand[i+k]=(char)0;
	
  XtFree(selectionstr);

  XtFree(*command);
  *command=newcommand;
}

void UserMenuSelectionSubst(char **command)
{
  char *newcommand;
  char *startstr;
  char *selectionstr;
  int i,j,k;
  long size;
  TViewNode *actual;

  if(SelectFileViewList==NULL) {
    selectionstr=XtMalloc(1);
    strcpy(selectionstr,"");
  } else {
    size=0;
    actual=SelectFileViewList;
    while(actual!=NULL) {
      size+=(strlen(actual->filenode->file.name)+1);
      actual=actual->next;
    }
    selectionstr = XtMalloc(size+1);

    actual=SelectFileViewList;
    strcpy(selectionstr, actual->filenode->file.name);
    actual=actual->next;
    while(actual!=NULL) {
      strcat(selectionstr, " ");
      strcat(selectionstr, actual->filenode->file.name);
      actual=actual->next;
    }
  }

  startstr = strstr(*command, "$SELECTION");
  if(startstr==NULL) {
    NonFatalError("command.c","UserMenuFileCommandRoutine","Cannot find $SELECTION field in User menu specification");
    return;
  }

  newcommand=XtMalloc(strlen(*command)+2+strlen(selectionstr)+1);

  /* create command string */
  for(i=0;i<((int)startstr-(int)(*command));i++) 
    newcommand[i]=(*command)[i];
  
  j=i;
  for(i=0;i<strlen(selectionstr);i++)
    newcommand[i+j]=selectionstr[i];

  k=i+j;
  for(i=0;i<(strlen(*command)-j-strlen("$SELECTION"));i++)
     newcommand[i+k]=(*command)[i+j+strlen("$SELECTION")];
  
  newcommand[i+k]=(char)0;
	
  XtFree(selectionstr);

  XtFree(*command);

  *command=newcommand;
  
}

void UserMenuRemoveKeyWord(char **command, char *keyword)
{
  char *newcommand;
  char *foundstr;
  int newlength;
  int i;

  newlength = strlen(*command)+1-strlen(keyword);

  newcommand= XtMalloc(sizeof(char)*newlength);
  
  foundstr=strstr(*command, keyword);
  if(foundstr==NULL)  {
    XtFree(newcommand);
    return;
  }

  for(i=0;i<(int)foundstr - (int) *command; i++) 
    newcommand[i]=(*command)[i];

  for(;i<newlength-1; i++)
    newcommand[i]=(*command)[i+strlen(keyword)];

  newcommand[i]=(char)0;

  XtFree(*command);
  
  *command=newcommand;
}


void UserMenuCommandRoutine(char *usercommand)
{
  TExecList execlist;
  char *command;

  command = XtMalloc(sizeof(char)*(strlen(usercommand)+1));
  strcpy(command, usercommand);

  if(strstr(command, "$SELECTION"))
    UserMenuSelectionSubst(&command);
  if(strstr(command, "$FILE"))
    UserMenuFileSubst(&command);
  if(strstr(command, "$SHELL")) {
    UserMenuRemoveKeyWord(&command,"$SHELL");
    if(strstr(command, "$NO_EXEC")) {
      UserMenuRemoveKeyWord(&command,"$NO_EXEC");
      ShellRoutine(command,0);
      if(command!=NULL)
	XtFree(command);
    } else
      ShellRoutine(command, 1);
    if(command!=NULL)
      XtFree(command);
  } else {
    execlist.interactive=0;
    execlist.nb_commands=1;
    execlist.headerstring = XtMalloc(strlen("User Command")+1);
    strcpy(execlist.headerstring, "User Command");
    execlist.command = (char **) XtMalloc(sizeof(char *));
    execlist.command[0]=command;
    
    execlist.usercommand=1;
    GeneralExecCommand2(execlist);
  }
}

void StatusStopButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;

  if(ChildExist) {

    w_temp=XtNameToWidget(W_StatusDialog, "*StatusStopButton");
    XtVaSetValues(w_temp, XmNsensitive, False, NULL);

    XmTextInsert(W_InputTextField, XmTextGetLastPosition(W_InputTextField), "\n --- Command aborted");

    NbCommandProcessed=ActualExecList.nb_commands-1;
    ChildErrorRaised=1;

  }
}

void StatusOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtUnmanageChild(W_StatusDialog);
}

void InitModuleCommand(void)
{
  Widget w_temp;

  w_temp=XtNameToWidget(W_StatusDialog, "*StatusStopButton");
  XtAddCallback(w_temp, XmNactivateCallback, StatusStopButtonCB, NULL);

  w_temp=XtNameToWidget(W_StatusDialog, "*StatusOkButton");
  XtAddCallback(w_temp, XmNactivateCallback, StatusOkButtonCB, NULL);
}
