#define FILEVIEW_H

/****************************************************************

    Copyright (c) 1996-1997 Fred Pesch

    This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License as published by the Free Software 
    Foundation (version 2 of the License).

    This program is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the 
    implied warranty of MERCHANTABILITY or FITNESS FOR A 
    PARTICULAR PURPOSE.  See the GNU General Public License 
    for more details.

    For a copy of the GNU General Public License, write to the 
    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
    MA 02139, USA.

****************************************************************/

#include <Xm/Xm.h>
#include <Xm/IconG.h>
#include <Xm/Container.h>
#include <stdlib.h>
#include <pwd.h>
#include <sys/types.h>
#include <time.h>

#include "filelist.h"
#include "error_handler.h"
#include "interface.h"
#include "config.h"
#include "pixmap.h"
#include "fof.h"
#include "sort.h"
#include "windows.h"
#include "dialogs.h"
#include "command.h"

#define FILE_NO_ICON -1

#define MAX_CELL_WIDTH 512
#define MAX_CELL_HEIGHT 256

struct DirIconNode {
  TDirNode *dir;
  int icon_index;

  struct DirIconNode *parent;
  
  int nb_children;
  struct DirIconNode **child_icon;
};

typedef struct DirIconNode TDirIconNode;

typedef struct {
  WidgetList w_icon;
  int nb_icons;
  int nb_icons_managed;
} TDirIconList;

extern TDirIconList DirIconList;

struct ViewNode {
  TFileNode *filenode;

  int icon_index;

  struct ViewNode *next;
};

typedef struct ViewNode TViewNode;

typedef struct {
  WidgetList w_icon;
  int nb_icons;
  int nb_icons_managed;
} TFileIconList;

extern TFileNode *ActualFileNodeList;
extern TViewNode *ActualFileViewList;
TViewNode *SelectFileViewList;

typedef enum { LARGE, SMALL, DETAIL } TViewMode;

TViewMode ActualViewMode;

TFileIconList FileIconList;

extern TDirIconNode *ActualIcon;
extern TDirIconNode *RootIcon;

extern TDirNode *ActualDir;

extern void GetLargestCellSize(TViewNode *filelist, int *width, int *height);
extern TDirIconNode *SearchIconNode(Widget w_icon, TDirIconNode *rootnode);

extern int LastLargestCellWidth;
extern int LastLargestCellHeight;

extern void RefreshFileList(void);
extern void MakeDirIconOneLevel(TDirIconNode *actualnode);

void GetSmallPixmap(TFileNode *node, Pixmap *pixmap)
{
  switch(node->file.type) {
  case DIRECTORY:
    if(node->file.readable) *pixmap = PIX_DIR_closed;
    else *pixmap = PIX_DIR_unreadable;
    break;
  case REGULAR_FILE:
    if(node->file.readable) *pixmap = PIX_FILE_SMALL_regular;
    else *pixmap = PIX_FILE_SMALL_unreadable;
    break;
  case EXECUTABLE_FILE:
    if(node->file.readable) *pixmap = PIX_FILE_SMALL_exec;
    else *pixmap = PIX_FILE_SMALL_unreadable;
    break;
  case SLINK_DIR:
    if(node->file.readable) *pixmap = PIX_FILE_SMALL_linkdir;
    else *pixmap = PIX_FILE_SMALL_unreadable;
    break;
  case SLINK_FILE:
    if(node->file.readable) *pixmap = PIX_FILE_SMALL_linkfile;
    else *pixmap = PIX_FILE_SMALL_unreadable;
    break;
  case FIFO:
  case SOCKET:
  case CHAR_DEVICE:
  case BLOCK_DEVICE:
    if(node->file.readable) *pixmap = PIX_FILE_SMALL_sys;
    else *pixmap = PIX_FILE_SMALL_unreadable;
    break;
  default:
    *pixmap = PIX_FILE_SMALL_unreadable;
    NonFatalError("fileview.c","CreateFileViewList","File type unknown (strange...)");
  }
}

void GetLargePixmap(TFileNode *node, Pixmap *pixmap)
{

  if(node->file.type==REGULAR_FILE || node->file.type==EXECUTABLE_FILE || node->file.type==SLINK_FILE) {
    *pixmap=GetFormatPixmap(node->file.name);
    if(*pixmap) return;
  }

  /* THEN USE DEFAULT ICONS */

  switch(node->file.type) {
  case DIRECTORY:
    if(node->file.readable) *pixmap = PIX_DIR_LARGE;
    else *pixmap = PIX_DIR_LARGE_unreadable;
    break;
  case REGULAR_FILE:
    if(node->file.readable) *pixmap = PIX_FILE_LARGE_regular;
    else *pixmap = PIX_FILE_LARGE_unreadable;
    break;
  case EXECUTABLE_FILE:
    if(node->file.executable) *pixmap = PIX_FILE_LARGE_exec;
    else if(node->file.readable) *pixmap = PIX_FILE_LARGE_regular;
    else *pixmap = PIX_FILE_LARGE_unreadable;
    break;
  case SLINK_FILE:
    if(node->file.readable) *pixmap = PIX_FILE_LARGE_linkfile;
    else *pixmap = PIX_FILE_LARGE_unreadable;
    break;
  case SLINK_DIR:
    if(node->file.readable) *pixmap = PIX_FILE_LARGE_linkdir;
    else *pixmap = PIX_FILE_LARGE_unreadable;
    break;  case FIFO:
  case SOCKET:
  case CHAR_DEVICE:
  case BLOCK_DEVICE:
    if(node->file.readable) *pixmap = PIX_FILE_LARGE_sys;
    else *pixmap = PIX_FILE_LARGE_unreadable;
    break;
  default:
    *pixmap = PIX_FILE_LARGE_unreadable;
    NonFatalError("fileview.c","CreateFileViewList","File type unknown (strange...)");
  }
}   

XmStringTable GetFileInfoStrings(TFileNode *filenode)
{
  XmStringTable strings;
  struct passwd *user_entry;
  char temp[256];

  strings = (XmStringTable) XtMalloc(sizeof(XmString)*5);
  if(strings == NULL) {
    NonFatalError("fileview.c","GetFileInfoStrings","Unable to Create details StringTable (XtMalloc failed)");
    return(NULL);
  }

  /* Get Owner string */
  user_entry = getpwuid(filenode->file.UID);
  if(user_entry==NULL) {
    NonFatalError("fileview.c","GetFileInfoStrings","Unable to get user infos (getpwuid failed)");
    XtFree((char *) strings);
    return(NULL);
  } 
  strings[0]= XmStringCreateSimple(user_entry->pw_name);
  if(strings[0]==NULL) {
    NonFatalError("fileview.c","GetFileInfoStrings","Unable to Create User name string");
    XtFree((char *) strings);
    return(NULL);
  }

  /* Get Access String */
  
  if(filenode->file.user & READ) temp[0]='r'; else temp[0]='-';
  if(filenode->file.user & WRITE) temp[1]='w'; else temp[1]='-';
  if(filenode->file.user & EXECUTE) temp[2]='x'; else temp[2]='-';

  if(filenode->file.group & READ) temp[3]='r'; else temp[3]='-';
  if(filenode->file.group & WRITE) temp[4]='w'; else temp[4]='-';
  if(filenode->file.group & EXECUTE) temp[5]='x'; else temp[5]='-';

  if(filenode->file.others & READ) temp[6]='r'; else temp[6]='-';
  if(filenode->file.others & WRITE) temp[7]='w'; else temp[7]='-';
  if(filenode->file.others & EXECUTE) temp[8]='x'; else temp[8]='-';

  temp[9]=(char) 0;

  strings[1]= XmStringCreateSimple(temp);
  if(strings[1]==NULL) {
    NonFatalError("dirtree.c","GetAccessString","Unable to Create Access String (XmStringCreateSimple failed)");
    XmStringFree(strings[0]);
    XtFree((char*)strings);
    return(NULL);
  }

  /* Get Date String */

  strftime(temp, sizeof(temp), "%b %d", localtime(&filenode->file.modiftime));
  
  strings[2]= XmStringCreateSimple(temp);
  if(strings[2]==NULL) {
    NonFatalError("dirtree.c","GetAccessString","Unable to Create Date String (XmStringCreateSimple failed)");
    XmStringFree(strings[0]);
    XmStringFree(strings[1]);
    XtFree((char*)strings);
    return(NULL);
  }
  /* Get Time String */

  strftime(temp, sizeof(temp), "%H:%M", localtime(&filenode->file.modiftime));
  
  strings[3]= XmStringCreateSimple(temp);
  if(strings[3]==NULL) {
    NonFatalError("dirtree.c","GetAccessString","Unable to Create Time String (XmStringCreateSimple failed)");
    XmStringFree(strings[0]);
    XmStringFree(strings[1]);
    XmStringFree(strings[2]);
    XtFree((char*)strings);
    return(NULL);
  }

  /* Get Size String */
  sprintf(temp, "%lu", filenode->file.size);
  strings[4]= XmStringCreateSimple(temp);
  if(strings[4]==NULL) {
    NonFatalError("dirtree.c","GetAccessString","Unable to Create Size String (XmStringCreateSimple failed)");
    XmStringFree(strings[0]);
    XmStringFree(strings[1]);
    XmStringFree(strings[2]);
    XmStringFree(strings[3]);
    XtFree((char*)strings);
    return(NULL);
  }
  
  return(strings);
}
 
TViewNode *CreateFileViewList(TFileNode *nodelist)
{
  TFileNode *actualnode;
  TViewNode *startview;
  TViewNode *actualview;
  Arg args[10];
  int nbargs;
  XmString string;
  Pixmap smallpixmap;
  Pixmap largepixmap;
  int nb_icons;
  int nb_files;
  char strtemp[100];
  XmStringTable infostrings;
  int exists;
  int iconlen;
  char *strtemp2;

  startview = (TViewNode *) XtMalloc(sizeof(TViewNode));
  if(startview == NULL) FatalError("fileview.c","CreateFileViewList","Unable to create View Node (XtMalloc failed)");
  
  startview->next=NULL;
  startview->filenode=nodelist;
  
  actualnode=nodelist;
  actualview=startview;
  nb_icons=0;
  nb_files=0;
  while(actualnode!=NULL) {

    /* Warning : Root Dir as no parent */
    if(actualnode->parent->parent==NULL && (strcmp(actualnode->file.name,"..")==0)) {
      actualview->icon_index=FILE_NO_ICON;
      nb_files++;
    }
    else {

      actualview->icon_index = nb_icons;
      
      /* Create small icon */
    
      nbargs=0;

      strtemp2=XtMalloc(strlen(actualnode->file.name)+4);
      exists=GetVarInt("IconTextMaxLength", &iconlen);
      if(exists && iconlen > 0 &&  strlen(actualnode->file.name)>iconlen) {
	strncpy(strtemp2,actualnode->file.name, iconlen-3);
	strtemp2[iconlen-3]=(char)0;
	strcat(strtemp2,"...");
      } else strcpy(strtemp2, actualnode->file.name);
      string= XmStringCreateSimple(strtemp2);
      XtFree(strtemp2);

      XtSetArg(args[nbargs], XmNlabelString, string);nbargs++;
      GetSmallPixmap(actualnode, &smallpixmap);
      XtSetArg(args[nbargs], XmNsmallIconPixmap, smallpixmap);nbargs++;
      XtSetArg(args[nbargs], XmNpositionIndex, actualnode->file.order);nbargs++;
      XtSetArg(args[nbargs], XmNentryParent, NULL);nbargs++;
      infostrings= GetFileInfoStrings(actualnode);
      if(infostrings!=NULL) {
	XtSetArg(args[nbargs], XmNdetail, infostrings);nbargs++;
	XtSetArg(args[nbargs], XmNdetailCount, 5);nbargs++;
      }
    
      /* Create large icon */
  
      GetLargePixmap(actualnode, &largepixmap);
      XtSetArg(args[nbargs], XmNlargeIconPixmap, largepixmap);nbargs++;
      XtSetArg(args[nbargs], XmNpositionIndex, actualnode->file.order);nbargs++;

      /* if we have to allocate a new widget */
      if(nb_icons>=FileIconList.nb_icons) {
	FileIconList.w_icon = (WidgetList) XtRealloc((char *) FileIconList.w_icon, sizeof(Widget)*(FileIconList.nb_icons+1));
	
	FileIconList.w_icon[nb_icons] = NULL;
	FileIconList.w_icon[nb_icons] = XmCreateIconGadget(W_ListFile, "FILEicon", args, nbargs);    
	if(FileIconList.w_icon[nb_icons] == NULL) FatalError("fileview.c","CreateFileViewList","Unable to create Icon (XmCreateIconGadget failed)");

	FileIconList.nb_icons++;

      } else /* if the widget already exists */
	XtSetValues(FileIconList.w_icon[nb_icons], args, nbargs);
      
      nb_icons++;
      nb_files++;
      FileIconList.nb_icons_managed++;
      
      if(infostrings!=NULL) {
	XmStringFree(infostrings[0]);
	XmStringFree(infostrings[1]);
	XmStringFree(infostrings[2]);
	XmStringFree(infostrings[3]);
	XmStringFree(infostrings[4]);
	XtFree((char *)infostrings);
      }
      
      XmStringFree(string);
    }
    
    actualnode=actualnode->next;
    if(actualnode!=NULL) {
      actualview->next = (TViewNode *) XtMalloc(sizeof(TViewNode));
      if(actualview->next == NULL) FatalError("fileview.c","CreateFileViewList","Unable to create View Node (XtMalloc failed)");
      actualview=actualview->next;
      actualview->next=NULL;
      actualview->filenode=actualnode;
    }
  }

  /* Manage the new icons */

  XtManageChildren(FileIconList.w_icon, FileIconList.nb_icons_managed);
  
  sprintf(strtemp, "  %d files read.", nb_files);
  WriteMessageLabel(strtemp);
  
  return(startview);
}

void DeleteFileViewList(TViewNode *start)
{
  TViewNode *temp;
  int i;

  i=FileIconList.nb_icons_managed;
  FileIconList.nb_icons_managed=0;
  XtUnmanageChildren(FileIconList.w_icon, i);
   
  while(start!=NULL) {
    temp=start->next;
    XtFree((char *)start);
    start=temp;
  }
}

TViewNode *GetSelectedViewNode(Widget w_selected, TViewNode *dirlist)
{
  TViewNode *actual;
  TViewNode *retnode;

  actual=dirlist;
  while(actual!=NULL && (FileIconList.w_icon[actual->icon_index]!=w_selected)) actual=actual->next;

  if(actual==NULL) return(NULL);

  retnode = (TViewNode *) XtMalloc(sizeof(TViewNode));
  retnode->filenode=actual->filenode;
  retnode->icon_index=actual->icon_index;
  return(retnode);
}


TViewNode *CreateSelectViewList(Widget w_selected, TViewNode *dirlist)
{
  TViewNode *start;

  start = GetSelectedViewNode(w_selected, dirlist);
  if(start==NULL) FatalError("fileview.c","CreateSelectViewList","Cannot find selected file");
  start->next=NULL;
  return(start);
}

TViewNode *AddSelectViewList(Widget w_selected, TViewNode *selectlist, TViewNode *dirlist)
{
  TViewNode *newnode;

  newnode = GetSelectedViewNode(w_selected, dirlist);
  if(newnode==NULL) FatalError("fileview.c","AddSelectViewList","Cannot find selected file");
  selectlist->next=newnode;
  newnode->next=NULL;
  return(newnode);
}

TViewNode *DeleteSelectViewList(TViewNode *startlist)
{
  TViewNode *temp;

  while(startlist!=NULL) {
    temp = startlist->next;
    XtFree((char *) startlist);
    startlist=temp;
  }

  return(NULL);
}

char *CreateSelectString(int nbselected, int nbdirs, int nbfiles, unsigned long size)
{
  char *selstring;
  int nbsystem;

  selstring=(char *) XtMalloc(sizeof(char)*256);
  strcpy(selstring,"cannot create info string");
  if(nbselected==0)
    sprintf(selstring, "no file selected");
  else if(nbselected==1) {
    if(nbfiles==1)
      sprintf(selstring, "1 file selected : %lu bytes in 1 file", size);
    else if(nbdirs==1)
      sprintf(selstring, "1 file selected : %lu bytes in 1 dir", size);
    else sprintf(selstring, "1 file selected (system)");
  } else if(nbselected>1) {
    nbsystem=nbselected-nbdirs-nbfiles;
    if(nbfiles!=0 && nbdirs!=0 && nbsystem!=0)
      sprintf(selstring, "%d files selected : %lu bytes in %d files, %d dirs, %d system files", nbselected, size, nbfiles, nbdirs, nbsystem);
    else if(nbfiles==0 && nbdirs!=0 && nbsystem!=0)
      sprintf(selstring, "%d files selected : %lu bytes in %d dirs, %d system files", nbselected, size, nbdirs, nbsystem);
    else if(nbfiles!=0 && nbdirs==0 && nbsystem!=0)
      sprintf(selstring, "%d files selected : %lu bytes in %d files, %d system files", nbselected, size, nbfiles, nbsystem);
    else if(nbfiles!=0 && nbdirs==0 && nbsystem==0)
      sprintf(selstring, "%d files selected : %lu bytes in %d files", nbselected, size, nbfiles);
    else if(nbfiles!=0 && nbdirs!=0 && nbsystem==0)
      sprintf(selstring, "%d files selected : %lu bytes in %d files, %d dirs", nbselected, size, nbfiles, nbdirs);
    else if(nbfiles==0 && nbdirs!=0 && nbsystem==0)
      sprintf(selstring, "%d files selected : %lu bytes in %d dirs", nbselected, size, nbdirs);
    else if(nbfiles==0 && nbdirs==0 && nbsystem!=0)
      sprintf(selstring, "%d files selected (system)", nbselected);
  }
  return(selstring);
}

void SelectionCB(Widget w_notebook, XtPointer app_data, XtPointer widget_data)
{
  XmContainerSelectCallbackStruct *infos;
  char *numstring;
  int nbdirs;
  int nbfiles;
  int i;
  unsigned long size;
  TViewNode *actual;

  infos = (XmContainerSelectCallbackStruct *) widget_data;
  /* Si aucun fichier selectionne */
  if(infos->selected_item_count==0) {
     if(SelectFileViewList!=NULL) 
      SelectFileViewList=DeleteSelectViewList(SelectFileViewList);
      numstring=CreateSelectString(0, 0, 0, 0);
  }
  /* Si un seul fichier */
  else if(infos->selected_item_count==1) {
    if(SelectFileViewList!=NULL) 
      SelectFileViewList=DeleteSelectViewList(SelectFileViewList);
    SelectFileViewList=CreateSelectViewList(infos->selected_items[0], ActualFileViewList);
    if(SelectFileViewList==NULL) FatalError("fileview.c","SelectionCB","Cannot create Selected Files List");
    nbdirs=0;
    nbfiles=0;
    size=0;
    switch(SelectFileViewList->filenode->file.type) {
    case DIRECTORY:
      nbdirs++;
      size+=SelectFileViewList->filenode->file.size;
      break;
    case REGULAR_FILE:
    case EXECUTABLE_FILE:
      nbfiles++;
      size+=SelectFileViewList->filenode->file.size;
      break;
    default:
    }
    numstring=CreateSelectString(1, nbdirs, nbfiles, size);
  }
      /* Sinon avec plusieurs fichiers */
  else {
    if(SelectFileViewList!=NULL) 
      SelectFileViewList=DeleteSelectViewList(SelectFileViewList);
    SelectFileViewList=CreateSelectViewList(infos->selected_items[0], ActualFileViewList);
    if(SelectFileViewList==NULL) FatalError("fileview.c","SelectionCB","Cannot create Selected Files List");
    nbdirs=0;
    nbfiles=0;
    size=0;
    actual=SelectFileViewList;
    for(i=1;i<=infos->selected_item_count;i++) {
      switch(actual->filenode->file.type) {
      case DIRECTORY:
      case SLINK_DIR:
	nbdirs++;
	size+=actual->filenode->file.size;
	break;
      case SLINK_FILE:
      case REGULAR_FILE:
      case EXECUTABLE_FILE:
	nbfiles++;
	size+=actual->filenode->file.size;
	break;
      default:
      }
      if(i<infos->selected_item_count)
	actual= AddSelectViewList(infos->selected_items[i], actual, ActualFileViewList);
    }
    numstring=CreateSelectString(infos->selected_item_count, nbdirs, nbfiles, size);
  }
  
  WriteMessageLabel(numstring);
  XtFree((char *)numstring);
}


/* this function returns the Dir Icon Widget to be selected in Tree container 
   when activated (dble-click) in file list container                  */
Widget GetDirIconToSelect(TViewNode *dirview, TDirIconNode *parent)
{
  TDirIconNode *actual=NULL;
  int i;
  int found;
  
  i = 0;
  found = 0;
  while(!found && i< parent->nb_children) {
    actual=parent->child_icon[i];
    if(strcmp(dirview->filenode->file.name, actual->dir->dir.name)==0) found=1;
    i++;
  }
  if(found) return(DirIconList.w_icon[actual->icon_index]);
  else return(NULL);
}

void ActionCB(Widget w_notebook, XtPointer app_data, XtPointer widget_data)
{
  XmContainerSelectCallbackStruct *infos;
  TViewNode *selected;
  Widget w_select;
  WidgetList wselected;
  TDirIconNode *founddir;

  infos = (XmContainerSelectCallbackStruct *) widget_data;

  if(infos->selected_item_count==1) {
    selected=GetSelectedViewNode(infos->selected_items[0], ActualFileViewList);
    /* if the object is readable */
    if(selected->filenode->file.readable) {
      /* if the user activates the parent directory */
      if(strcmp(selected->filenode->file.name,"..")==0) {
	wselected = (WidgetList) XtMalloc(sizeof(Widget));
	wselected[0] = DirIconList.w_icon[ActualIcon->parent->icon_index];
	XtVaSetValues(W_TreeContainer, XmNselectedObjectCount, 1, XmNselectedObjects, wselected, NULL, NULL);
	founddir = (TDirIconNode *) SearchIconNode(wselected[0], RootIcon);
	
	if(founddir!=NULL && founddir->dir->dir.readable==1) {
	  ActualIcon = founddir;
	  ActualDir = ActualIcon->dir;
	  
	  RefreshFileList();
	  InitRefreshProc();
	  
	  ChangeWindowProperty();
	  
	  XmScrollVisible(W_ScrollTree, wselected[0], 10, 10); 
	}
	XtFree((char *) wselected);
	/* if the user activates another directory */
      } else if(selected->filenode->file.type==DIRECTORY || selected->filenode->file.type==SLINK_DIR) {
	wselected = (WidgetList) XtMalloc(sizeof(Widget));
	/* if the actual dir is closed, open it first */
	/*	if(ActualIcon==RootIcon) {
		XtVaSetValues(ActualIcon->w_icon, XmNoutlineState, XmEXPANDED, NULL);
		fprintf(stderr,"\n you want to open the root dir\n\n");
		}
		else*/
	if(!ActualDir->dir.opened) {	  
	  w_select = DirIconList.w_icon[ActualIcon->icon_index];
	  founddir = SearchIconNode(w_select, RootIcon);
	  
	  if(founddir!=NULL && founddir->dir->dir.readable==1 && founddir->dir->dir.opened==0) {
	    ActualIcon = founddir;
	    ActualDir = ActualIcon->dir;
	    
	    MakeDirOneLevel(ActualDir);
	    MakeDirIconOneLevel(ActualIcon);

	  }
	}
	/* now selects the activated dir in tree container */
	wselected[0] = GetDirIconToSelect(selected, ActualIcon);
	if(wselected[0]==NULL) {
	  fprintf(stderr,"Dir Icon not found !!\n\n");
          XtFree((char *) wselected);
	  return;
	}
	XtVaSetValues(W_TreeContainer, XmNselectedObjectCount, 1, XmNselectedObjects, wselected, NULL);
	founddir = (TDirIconNode *) SearchIconNode(wselected[0], RootIcon);
	
	if(founddir!=NULL && founddir->dir->dir.readable==1) {
	  ActualIcon = founddir;
	  ActualDir = ActualIcon->dir;
	  
	  RefreshFileList();
	  InitRefreshProc();

	  ChangeWindowProperty();
	  
	  XmScrollVisible(W_ScrollTree, wselected[0], 10, 10); 
	}
	XtFree((char *) wselected);
      } else if(selected->filenode->file.executable)
	FileExecRoutine();
      else FileViewCommandRoutine();
    }
  }
}

void GetLargestCellSize(TViewNode *filelist, int *width, int *height)
{
  TViewNode *actual;
  int temp_w, temp_h;

  *width=0;
  *height=0;
  actual=filelist;
  while(actual!=NULL) {
    if(actual->icon_index!=-1 && FileIconList.w_icon[actual->icon_index]!=NULL) {
      XtVaGetValues(FileIconList.w_icon[actual->icon_index], XmNwidth, &temp_w, XmNheight, &temp_h, NULL, NULL);

      if(temp_w>*width) *width=temp_w;
      if(temp_h>*height) *height=temp_h;
    }

    actual=actual->next;
  }
}

Dimension GetLargeCellsHeight(int container_width, int cell_width, int cell_height)
{
  int nb_icons, nb_icons_by_row;
  Dimension height;

  nb_icons=FileIconList.nb_icons_managed;
  nb_icons_by_row = (container_width / cell_width) +1;

  height=(nb_icons/nb_icons_by_row)*cell_height;
  if(height<cell_height) height=cell_height;
  return(height);
}
  

void FixFileViewSize(Widget parent, XtPointer p1, XtPointer p2) 
{
  Dimension width, height, container_w, container_h;
  int c_width, c_height;
  Dimension new_height;
  int recalc_height;
  
  recalc_height=0;
  if(ActualViewMode!=DETAIL) {
    GetLargestCellSize(ActualFileViewList, &c_width, &c_height);
    if(c_width>0 && c_width<MAX_CELL_WIDTH && c_height>0 && c_height<MAX_CELL_HEIGHT) recalc_height=1;
    if(!recalc_height) {
      if(ActualViewMode==LARGE) 
	XtVaGetValues(W_ListFile, XmNlargeCellWidth, &c_width, XmNlargeCellHeight, &c_height, NULL);
      else
	XtVaGetValues(W_ListFile, XmNsmallCellWidth, &c_width, XmNsmallCellHeight, &c_height, NULL);
	

      if(c_width>0 && c_width<MAX_CELL_WIDTH && c_height>0 && c_height<MAX_CELL_HEIGHT) recalc_height=1;
    }
    if(!recalc_height) {
      c_width=LastLargestCellWidth;
      c_height=LastLargestCellHeight;
      if(c_width>0 && c_width<MAX_CELL_WIDTH && c_height>0 && c_height<MAX_CELL_HEIGHT) recalc_height=1;
    }
    if(FileIconList.nb_icons_managed==0) recalc_height=0;
  }
  
  /* Get the correct dimensions of the clipwindow */
  XtVaGetValues(XtParent(W_ListFile), XmNwidth, &width, XmNheight, &height, NULL, NULL);

  /* Get the correct dimensions of the container */
  XtVaGetValues(W_ListFile, XmNwidth, &container_w,
                XmNheight, &container_h, NULL, NULL);

  /* if(container_w < width)*/
  if(ActualViewMode!=DETAIL)
    XtVaSetValues(W_ListFile, XmNwidth, width, NULL, NULL);

  if(recalc_height) {
    new_height=GetLargeCellsHeight(width, c_width, c_height);

    XtVaSetValues(W_ListFile, XmNheight, new_height, NULL);
  }

  /* Get the correct dimensions of the container */
  XtVaGetValues(W_ListFile, XmNwidth, &container_w,
                XmNheight, &container_h, NULL, NULL);

  if (container_h < height) 
    XtVaSetValues(W_ListFile, XmNheight, height, NULL, NULL);
  
  if(recalc_height) SortRoutine();
}

void SelectViewMode(TViewMode mode)
{
  if(mode==ActualViewMode) return;
  switch(mode) {
  case LARGE:
    ActualViewMode=LARGE;
    XtVaSetValues(W_ListFile, XmNlayoutType, XmSPATIAL, XmNentryViewType, XmLARGE_ICON, NULL);
    RefreshFileList();
    break;
  case SMALL:
    ActualViewMode=SMALL;
    XtVaSetValues(W_ListFile, XmNlayoutType, XmSPATIAL, XmNentryViewType, XmSMALL_ICON, NULL);
    RefreshFileList();
    break;
  case DETAIL:
    ActualViewMode=DETAIL;
    XtVaSetValues(W_ListFile, XmNlayoutType, XmDETAIL, XmNentryViewType, XmSMALL_ICON, NULL);
    RefreshFileList();
    break;
  }

}

	 
void InstallFileViewCB(void)
{
  XtAddCallback(W_ListFile, XmNselectionCallback, SelectionCB, NULL);

  XtAddCallback(W_ListFile, XmNdefaultActionCallback, ActionCB, NULL);

  XtAddCallback(XtParent(W_ListFile), XmNresizeCallback, FixFileViewSize, NULL);

}

void SelectDefaultViewMode(void)
{
  char *string;
  Widget w_temp;

  GetVarString("DefaultViewMode", &string);
  if(strcmp(string,"LARGE")==0) {
    SelectViewMode(LARGE);
    w_temp=XtNameToWidget(W_MainWindow, "*ViewLargeButton");
    XmToggleButtonSetState(w_temp, True, True);
  }
  if(strcmp(string,"SMALL")==0) {
    SelectViewMode(SMALL);
    w_temp=XtNameToWidget(W_MainWindow, "*ViewSmallButton");
    XmToggleButtonSetState(w_temp, True, True);
  }

    if(strcmp(string,"DETAIL")==0) {
    SelectViewMode(DETAIL);
    w_temp=XtNameToWidget(W_MainWindow, "*ViewDetailButton");
    XmToggleButtonSetState(w_temp, True, True);
  }

  GetVarString("DefaultSortMode", &string);
  if(strcmp(string,"TYPE")==0) {
    ActualSortType=SORT_BY_TYPE;
    w_temp=XtNameToWidget(W_MainWindow, "*SortByTypeButton");
    XmToggleButtonSetState(w_temp, True, True);
  }
    
  if(strcmp(string,"NAME")==0) {
    ActualSortType=SORT_BY_NAME;
    w_temp=XtNameToWidget(W_MainWindow, "*SortByNameButton");
    XmToggleButtonSetState(w_temp, True, True);
  }

  if(strcmp(string,"SIZE")==0) {
    ActualSortType=SORT_BY_SIZE;  
    w_temp=XtNameToWidget(W_MainWindow, "*SortBySizeButton");
    XmToggleButtonSetState(w_temp, True, True);
  }
}

void InitModuleFileView(void)
{

  SelectFileViewList=NULL;
  InstallFileViewCB();

  ActualFileViewList=NULL;
  ActualFileNodeList=NULL;

  FileIconList.nb_icons=0;
  FileIconList.nb_icons_managed=0;

}

